package com.tf.drawing
{
	
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	import mx.core.Container;
	import mx.core.UIComponent;
	import mx.events.FlexEvent;
	
	public class DrawableOverlay extends UIComponent
	{
		
		private var lines:Array = new Array();
		private var curentLine:Line = null;
		
		
		public function DrawableOverlay()
		{
			super();
			
			this.x = 0;
			this.y = 0;
			this.percentWidth = 100;
			this.percentHeight = 100;
			
			this.mouseEnabled = false; // prevent catch mouse events too early
			
			this.addEventListener(FlexEvent.ADD, onAdded);
		}
		
		
		/** Add mouse listeners to detect drawing */
		private function onAdded(e:FlexEvent):void {
			this.parent.addEventListener(MouseEvent.MOUSE_DOWN, downEvent);
			this.parent.addEventListener(MouseEvent.MOUSE_UP, upEvent);
			this.parent.addEventListener(MouseEvent.MOUSE_OUT, upEvent);
		}
		
		
		/** Start a new line when the mouse button is pressed */
		private function downEvent(event:MouseEvent):void
		{
			// Get mouse coordinates
			var target:Container = event.target as Container;
			if(!target) return;
			var point:Point = target.localToGlobal(new Point(event.localX, event.localY));
			point = this.globalToLocal(point);
			
			// Create a new line
			curentLine = new Line();
			curentLine.push(point);
			lines.push( curentLine );
			
			// Enforce max lines limit
			if(lines.length > 2)
				lines.shift();
			
			// Add move listener to track this line
			this.parent.addEventListener(MouseEvent.MOUSE_MOVE, moveEvent);
			
			// Cause component to redraw
			this.invalidateDisplayList();
		}
		
		
		/** Detect mouse movements to extend the line */
		private function moveEvent(event:MouseEvent):void
		{	
			// Get mouse coordinates
			var target:Container = event.target as Container;
			if(!target) return;
			var point:Point = target.localToGlobal(new Point(event.localX, event.localY));
			point = this.globalToLocal(point);
			
			// Add point to the line
			curentLine.push(point); 
			
			// Cause component to redraw
			this.invalidateDisplayList();
		}
		
		
		/** Stop drawing this line */
		private function upEvent(event:MouseEvent):void
		{
			this.parent.removeEventListener(MouseEvent.MOUSE_MOVE, moveEvent);
		}
		
		
		/** Draw each line onto this component */
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number) : void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			this.graphics.clear();
			
			for( var i:int = 0; i<lines.length; i++ )
			{
				var line:Line = lines[i];
				line.draw(this.graphics);
			}
		}
	}
}